@@ -11,7 +11,7 @@ get_model_function <- function(model) {
     inherits(model, "polr") ~ "glm",
     inherits(model, "vgam") ~ "glm",
     inherits(model, "vglm") ~ "glm",
-    inherits(model, "glm") ~ "glm",
+    inherits(model, c("logistf", "glm")) ~ "glm",
     inherits(model, "gls") ~ "lm",
     inherits(model, "gee") ~ "lm",
     inherits(model, "plm") ~ "lm",
@@ -49,6 +49,7 @@ get_predict_function <- function(model) {
     inherits(model, "negbin") ~ "glm.nb",
     inherits(model, "vgam") ~ "vgam",
     inherits(model, "vglm") ~ "vglm",
+    inherits(model, "logistf") ~ "logistf",
     inherits(model, "glm") ~ "glm",
     inherits(model, "lm") ~ "lm",
     inherits(model, "betareg") ~ "betareg",
